open Okauth_crypto

module Types = struct
  type user = {
    id: String.t;
    password_hash: String.t;
    public_key: Public_key.t;
    secret_key: Secret_key.t;
    name: String.t;
  } [@@deriving equal]
end

module type User = sig
  include module type of Types

  type t = user [@@deriving equal]

  (** Creates new user, generates its public and secret keys and
      persists in database *)
  val create:
    Sqlite3.db ->
    id: string ->
    password: string ->
    name: string ->
    t

  (** Removes user under given ID.
      Does nothing if user does not exist.*)
  val remove:
    Sqlite3.db ->
    id: string ->
    unit

  (** Retrieves all users from database *)
  val find_all:
    Sqlite3.db ->
    t array

  (** Retrieves one user by its ID *)
  val find_by_id:
    Sqlite3.db ->
    id: string ->
    t option

  (** Changes user's password.
      Does nothing if user does not exist. *)
  val change_password:
    Sqlite3.db ->
    id: string ->
    new_password: string ->
    unit
end
