module Bindings (F : Cstubs.FOREIGN) = struct
  include Ctypes

  let foreign = F.foreign
  let (@->) = F.(@->)
  let returning = F.returning

  let crypto_box_publickeybytes = foreign "crypto_box_publickeybytes" (void @-> returning int)

  let crypto_box_secretkeybytes = foreign "crypto_box_secretkeybytes" (void @-> returning int)

  let crypto_box_seedbytes = foreign "crypto_box_seedbytes" (void @-> returning int)

  let crypto_box_noncebytes = foreign "crypto_box_noncebytes" (void @-> returning int)

  let crypto_box_macbytes = foreign "crypto_box_macbytes" (void @-> returning int)

  let randombytes_buf = foreign "randombytes_buf" (
      ptr char @-> (* output array *)
      int @-> (* size *)
      returning void)

  let crypto_box_keypair = foreign "crypto_box_keypair" (
      ptr char @-> (* public key output array *)
      ptr char @-> (* secret key output array *)
      returning void)

  let crypto_box_seed_keypair = foreign "crypto_box_seed_keypair" (
      ptr char @-> (* public key output array *)
      ptr char @-> (* secret key output array *)
      ptr char @-> (* seed *)
      returning void)

  let crypto_box_easy = foreign "crypto_box_easy" (
      ptr char @-> (* ciphertext output array *)
      ptr char @-> (* plaintext *)
      int @-> (* plaintext size *)
      ptr char @-> (* nonce *)
      ptr char @-> (* public key *)
      ptr char @-> (* secret key *)
      returning int)

  let crypto_box_open_easy = foreign "crypto_box_open_easy" (
      ptr char @-> (* plaintext output array *)
      ptr char @-> (* ciphertext *)
      int @-> (* ciphertext size *)
      ptr char @-> (* nonce *)
      ptr char @-> (* public key *)
      ptr char @-> (* secret key *)
      returning int)

  let crypto_pwhash_strbytes = foreign "crypto_pwhash_strbytes" (void @-> returning int)

  let crypto_pwhash_opslimit_sensitive = foreign "crypto_pwhash_opslimit_sensitive" (void @-> returning int)

  let crypto_pwhash_memlimit_sensitive = foreign "crypto_pwhash_memlimit_sensitive" (void @-> returning int)

  let crypto_pwhash_str = foreign "crypto_pwhash_str" (
      ptr char @-> (* hash output array *)
      ptr char @-> (* password *)
      int @-> (* password size *)
      int @-> (* operations limit *)
      int @-> (* memory limit *)
      returning int)

  let crypto_pwhash_str_verify = foreign "crypto_pwhash_str_verify" (
      ptr char @-> (* hash *)
      ptr char @-> (* password *)
      int @-> (* password size *)
      returning int)

  let crypto_sign_publickeybytes = foreign "crypto_sign_publickeybytes" (void @-> returning int)

  let crypto_sign_secretkeybytes = foreign "crypto_sign_secretkeybytes" (void @-> returning int)

  let crypto_sign_bytes = foreign "crypto_sign_bytes" (void @-> returning int)

  let crypto_sign_keypair = foreign "crypto_sign_keypair" (
      ptr char @-> (* public key output *)
      ptr char @-> (* secret key output *)
      returning void)

  let crypto_sign_detached = foreign "crypto_sign_detached" (
      ptr char @-> (* signrature output *)
      ptr ullong @-> (* signature size, could be ignored *)
      ptr char @-> (* message *)
      int @-> (* message size *)
      ptr char @-> (* secret key *)
      returning int)

  let crypto_sign_verify_detached = foreign "crypto_sign_verify_detached" (
      ptr char @-> (* signature *)
      ptr char @-> (* message *)
      int @-> (* message size *)
      ptr char @-> (* public key *)
      returning int)
end
